function [ communitiesPapers ] = CombineCommOneSide( communities, communitiesPapers, connectionSubGraph )
%This function is used to combine entire communities that are similar.
%This combines communities on one side of the bipartite graph

    newCommunities = communities;
    
    %similarity between different communities
    correlationMatrix = zeros(length(communities));
    
    %Do nothing if there is only one community
    if length(communities) == 1
        return
    end
    
    %AvgIn represents the average closeness within a community
    avgIn = zeros(length(communities),1);
    
    %Compute avgIn for all of the communities
    for i = 1:length(communities)
        avgIn(i)= averageConnections(i, i, communities, communitiesPapers, connectionSubGraph);
    end
    
    %Compute the pairwise similarity between all of the communities
    for i = 1:length(communities)
         avgI = avgIn(i);
         
         for j = 1:length(communities)
            avgJ = avgIn(j);
            
            %Determine the avg similarity within and outside of the
            %community
            [avgIJ, avgIJOut] = averageConnections(i, j, communities, communitiesPapers, connectionSubGraph);
            
            %Side case to ignore which happens if one community
            if(isnan(avgIJOut))
                avgIJOut = Inf;
            end
            
            %Compute the similarity of the new connections from I to J
            subMatrixIJ = connectionSubGraph(communitiesPapers == communities(i), communitiesPapers == communities(j));
            [nIn1, nIn2] = size(subMatrixIJ);            
            avgInBetweenConnection = sum(sum(subMatrixIJ))/(nIn1*nIn2);
            
            %Only consider combining communities if they are sufficiently
            %similar to each other
            if(avgInBetweenConnection >= 1/2*min(avgI, avgJ))    %Enforces that there are connections between the different communities
                correlationMatrix(i, j) = avgIJ - avgIJOut;     %current best
            else
                correlationMatrix(i, j) = -Inf;
            end

        end
    end
    
    
    %Combine communities based on which community a given community feels
    %closest to
    for i = 1:length(communities)
        tmp = max(correlationMatrix(i,:));
        if tmp > 0
            newCommunities(i) = newCommunities(find(correlationMatrix(i,:) == max(correlationMatrix(i,:)),1));
        end
    end
    
    %Update the corresponding nodes to have the new communities
    for i = 1:length(communities)
       communitiesPapers(communitiesPapers == communities(i)) = newCommunities(i); 
    end
end


function [avgConnectionIn, avgConnectionOut] = averageConnections(i, j, communities, communitiesPapers, connectionSubGraph)
    tmp = communitiesPapers == communities(j) | communitiesPapers == communities(i);
    subMatrix = connectionSubGraph(tmp,tmp);
    subMatrixOut = connectionSubGraph(tmp,~tmp);

    n = size(subMatrix, 1);
    [nout1, nout2] = size(subMatrixOut);

    if n == 1
        avgConnectionIn = subMatrix;
    else
        subMatrix(1:n+1:end) = 0;
        avgConnectionIn = sum(sum(subMatrix))/(n*(n-1));
    end
    avgConnectionOut = sum(sum(subMatrixOut))/(nout1*nout2);
end