function [map,map_mu] = curvature_guided_Teichmuller_map(v1,f1,bdy1,blm1,ilm1,v2,f2,bdy2,blm2,ilm2,epsilon)

% Compute the curvature guided landmark-matching Teichmuller map from shape 1 to shape 2
%
% Input:
% v1: nv1 x 2 vertex coordinates of shape 1
% f1: nf1 x 3 triangulations of shape 1
% bdy1: indices of boundary nodes of shape 1
% blm1: indices of boundary landmarks of shape 1
% ilm1: indices of interior landmarks of shape 1
% v2: nv1 x 2 vertex coordinates of shape 2
% f2: nf1 x 3 triangulations of shape 2
% bdy2: indices of boundary nodes of shape 2
% blm2: indices of boundary landmarks of shape 2
% ilm2: indices of interior landmarks of shape 2
% epsilon: (optional) stopping parameter for QC Iteration
%
% Output:
% map: nv1 x 2 vertex coordinates of the resulting Teichmuller map
% map_mu: nf1 x 1 norm of the Beltrami coefficient on each face 
% 
% Copyright (c) 2018, Gary P. T. Choi
% https://scholar.harvard.edu/choi

%%
% rearrange bdy according to blm
[~,bind1] = intersect(bdy1,blm1);
[~,bind2] = intersect(bdy2,blm2);
bdy1 = bdy1([min(bind1):length(bdy1),1:min(bind1)-1]);
bdy2 = bdy2([min(bind2):length(bdy2),1:min(bind2)-1]);
   
if length(v1) > size(v1,1)
    v1 = v1';
end
if length(v2) > size(v2,1)
    v2 = v2';
end
if size(v1,2) > 2
    v1 = v1(:,1:2);
end
if size(v2,2) > 2
    v2 = v2(:,1:2);
end
if length(bdy1) > size(bdy1,1)
    bdy1 = bdy1';
end
if length(bdy2) > size(bdy2,1)
    bdy2 = bdy2';
end
if length(blm1) > size(blm1,1)
    blm1 = blm1';
end
if length(blm2) > size(blm2,1)
    blm2 = blm2';
end
if length(ilm1) > size(ilm1,1)
    ilm1 = ilm1';
end
if length(ilm2) > size(ilm2,1)
    ilm2 = ilm2';
end
     
curv1 = abs(compute_curvature(v1(bdy1,1),v1(bdy1,2)));
curv2 = abs(compute_curvature(v2(bdy2,1),v2(bdy2,2)));

% to avoid exactly zero curvature that affects the matching step, add a small value 
regularization = 1e-8;
curv1 = curv1 + regularization;
curv2 = curv2 + regularization;

% compute boundary edge lengths
bdyedgelength1 = abs(v1(bdy1,1) - v1(bdy1([length(bdy1),1:end-1]),1) + 1i*(v1(bdy1,2) - v1(bdy1([length(bdy1),1:end-1]),2)));
bdyedgelength2 = abs(v2(bdy2,1) - v2(bdy2([length(bdy2),1:end-1]),1) + 1i*(v2(bdy2,2) - v2(bdy2([length(bdy2),1:end-1]),2)));
bdyedgelength1 = (bdyedgelength1 + bdyedgelength1([2:end,1]))/2;
bdyedgelength2 = (bdyedgelength2 + bdyedgelength2([2:end,1]))/2;

weighted_curv1 = (curv1.*bdyedgelength1) / sum(curv1.*bdyedgelength1);
weighted_curv2 = (curv2.*bdyedgelength2) / sum(curv2.*bdyedgelength2);

% discrete curvature computed using too few points is highly discontinuous,
% use convolution to get a smooth representation
for kkk = 1:15
    weighted_curv1 = conv(weighted_curv1([end-19:end, 1:end, 1:20]), [1 1 1 1 1], 'same')/5;
    weighted_curv1 = weighted_curv1(21:end-20);
    weighted_curv2 = conv(weighted_curv2([end-19:end, 1:end, 1:20]), [1 1 1 1 1], 'same')/5;
    weighted_curv2 = weighted_curv2(21:end-20);
end


bdytarget = zeros(length(bdy1),2);
% find the location of blm in bdy
index1 = zeros(length(blm1),1);
for i = 1:length(blm1)
    pos = find(bdy1 == blm1(i));
    index1(i) = pos;
end
bdytarget(index1,:) = v2(blm2,1:2);
if index1(end) == max(index1)
    index1 = [index1; length(bdy1)];
else index1 = [index1; index1(1)-1];
end

index2 = zeros(length(blm2),1);
for i = 1:length(blm2)
    pos = find(bdy2 == blm2(i));
    index2(i) = pos;
end
if index2(end) == max(index2)
    index2 = [index2; length(bdy2)];
else index2 = [index2; index2(1)-1];
end

for i = 2:length(index1)
    if index1(i-1)>index1(i)
        index1(i) = index1(i) + length(bdy1);
    end
end
for i = 2:length(index2)
    if index2(i-1)>index2(i)
        index2(i) = index2(i) + length(bdy2);
    end
end

weighted_curv1 = [weighted_curv1;weighted_curv1];
weighted_curv2 = [weighted_curv2;weighted_curv2];

bdyedgelength1 = [bdyedgelength1;bdyedgelength1];
bdyedgelength2 = [bdyedgelength2;bdyedgelength2];

%% use SRSF on accumulated curvature
for i = 2:length(index1)
    %%
    segmentlength1 = bdyedgelength1((index1(i-1)+1):index1(i));
    segmentlength2 = bdyedgelength2((index2(i-1)+1):index2(i));
    segment1 = mod((index1(i-1)+1):index1(i),length(bdy1));
    segment1(segment1 == 0) = length(bdy1);
    segment2 = mod((index2(i-1)+1):index2(i),length(bdy2));
    segment2(segment2 == 0) = length(bdy2);
    
    segmentcurv1 = weighted_curv1((index1(i-1)+1):index1(i));
    segmentcurv2 = weighted_curv2((index2(i-1)+1):index2(i));
    
    if ~isempty(segmentlength1) && ~isempty(segmentlength2)
        %% segment ratio (x-axis)
        segmentratio1 = zeros(length(segmentlength1),1);
        segmentratio2 = zeros(length(segmentlength2),1);
        segmentratio1(1) = segmentlength1(1)/sum(segmentlength1);
        segmentratio2(1) = segmentlength2(1)/sum(segmentlength2);
        for j = 2:length(segmentratio1)
            segmentratio1(j) = segmentratio1(j-1) + segmentlength1(j)/sum(segmentlength1);
        end
        for j = 2:length(segmentratio2)
            segmentratio2(j) = segmentratio2(j-1) + segmentlength2(j)/sum(segmentlength2);
        end    
        segmentratio1 = [0;segmentratio1];
        segmentratio2 = [0;segmentratio2];
        
        %  segment curvature (y-axis)
        accumulated_curv_ratio1 = zeros(length(segmentcurv1),1);
        accumulated_curv_ratio2 = zeros(length(segmentcurv2),1);
        accumulated_curv_ratio1(1) = segmentcurv1(1)/sum(segmentcurv1);
        accumulated_curv_ratio2(1) = segmentcurv2(1)/sum(segmentcurv2);
        for j = 2:length(accumulated_curv_ratio1)
            accumulated_curv_ratio1(j) = accumulated_curv_ratio1(j-1) + segmentcurv1(j)/sum(segmentcurv1);
        end
        for j = 2:length(accumulated_curv_ratio2)
            accumulated_curv_ratio2(j) = accumulated_curv_ratio2(j-1) + segmentcurv2(j)/sum(segmentcurv2);
        end    
        accumulated_curv_ratio1 = [0;accumulated_curv_ratio1];
        accumulated_curv_ratio2 = [0;accumulated_curv_ratio2];

        
        if length(segmentratio1)>2 && length(segmentratio2)>2
            %% SRVF
            % warpping result gives optimal correspondence between
             % accumulated_curv_ratio1 and accumulated_curv_ratio2

            ratio = (0:0.01:1)';
            segmentratio1(1) = 0; segmentratio2(1) = 0; ratio(1) = 0; accumulated_curv_ratio1(1) = 0; accumulated_curv_ratio2(1) = 0;
            segmentratio1(end) = 1; segmentratio2(end) = 1; ratio(end) = 1; accumulated_curv_ratio1(end) = 1; accumulated_curv_ratio2(end) = 1;
            interp_seg1 = interp1(segmentratio2, accumulated_curv_ratio2, ratio,'linear');
            interp_seg2 = interp1(segmentratio1, accumulated_curv_ratio1, ratio,'linear');
            [warpping_result,~] = mainWarpingWithReference(ratio,[interp_seg2,interp_seg1],0);
             
             %% project curv1 onto warpping result, find the corresponding x-coords
             
             warpped_segmentratio1 = interp1(warpping_result, ratio, accumulated_curv_ratio1,'linear');

        end
        %% interpolation according to warpped segment ratio
        ratio1 = warpped_segmentratio1(2:end);
        ratio2 = segmentratio2(2:end);
        
        if i~=length(index1)
            %%
            for j = 1:length(segment1)-1
                %%
                list = find(ratio2 > ratio1(j));
                interval = [list(1)-1, list(1)];
                if interval(1) ~= 0                
                    r = (ratio1(j) - ratio2(interval(1))) /(ratio2(interval(2)) - ratio2(interval(1)));            
                    bdytarget(segment1(j),:) = v2(bdy2(segment2(interval(1))),:) + r* (v2(bdy2(segment2(interval(2))),:) - v2(bdy2(segment2(interval(1))),:));
                else
                    r = (ratio1(j) - 0) /(ratio2(interval(2)) - 0);            
                    bdytarget(segment1(j),:) = v2(bdy2(index2(i-1)),:) + r* (v2(bdy2(segment2(interval(2))),:) - v2(bdy2(index2(i-1)),:));
                end
            end
        else 
            for j = 1:length(segment1)
                %%
                list = find(ratio2 > ratio1(j));
                if isempty(list)
                    interval = [length(segment2),1];
                else
                    interval = [list(1)-1, list(1)];
                end
                if interval(1) ~= 0                
                    r = (ratio1(j) - ratio2(interval(1))) /(ratio2(interval(2)) - ratio2(interval(1)));            
                    bdytarget(segment1(j),:) = v2(bdy2(segment2(interval(1))),:) + r* (v2(bdy2(segment2(interval(2))),:) - v2(bdy2(segment2(interval(1))),:));
                else
                    r = (ratio1(j) - 0) /(ratio2(interval(2)) - 0); 
                    temp = mod(index2(i-1),length(bdy2));
                    if temp == 0
                        temp = length(bdy2);
                    end
                    bdytarget(segment1(j),:) = v2(bdy2(temp),:) + r* (v2(bdy2(segment2(interval(2))),:) - v2(bdy2(temp),:));
                end
            end
        end
    end
end

%% Landmark matching Teichmuller map with the above boundary correspondence
point = [bdy1;ilm1];
inttarget = v2(ilm2,:); 

target = [bdytarget; inttarget];
if exist('epsilon','var')
    [map,map_mu] = QCiteration(v1',f1',point,target,epsilon);
else
    [map,map_mu] = QCiteration(v1',f1',point,target);
end
map = map';
