% Demo: Computing a curvature-guided, landmark-matching Teichmller mapping between two Drosophila wings
%
% Copyright (c) 2018, Gary P. T. Choi
% https://scholar.harvard.edu/choi

addpath(genpath('.'));

% load data
data1 = load('084-punalua F2a z16 s70.mat');
v1 = data1.v; f1 = data1.f; bdy1 = data1.bdy; blm1 = data1.blm; ilm1 = data1.ilm;
data2 = load('040-silvestris Mob z16 s75.mat');
v2 = data2.v; f2 = data2.f; bdy2 = data2.bdy; blm2 = data2.blm; ilm2 = data2.ilm;
I1 = im2double(imread('punalua F2a z16 s70.tif')); 
I2 = im2double(imread('silvestris Mob z16 s75.tif'));

% plot the source wing with landmarks
figure;
patch('Faces',f1,'Vertices',v1,'FaceColor','None','EdgeColor','k');
axis equal tight;
hold on;
plot(v1(blm1,1),v1(blm1,2),'ro','MarkerFaceColor','r');
plot(v1(ilm1,1),v1(ilm1,2),'ro','MarkerFaceColor','r');
title('Source');
set(gca,'FontSize',15);


% plot the target wing with landmarks
figure;
patch('Faces',f2,'Vertices',v2,'FaceColor','None','EdgeColor','k');
axis equal tight;
hold on;
plot(v2(blm2,1),v2(blm2,2),'go','MarkerFaceColor','g');
plot(v2(ilm2,1),v2(ilm2,2),'go','MarkerFaceColor','g');
title('Target');
set(gca,'FontSize',15);

% landmark-matching curvature-guided Teichmuller mapping
[map,map_mu] = curvature_guided_Teichmuller_map(v1,f1,bdy1,blm1,ilm1,v2,f2,bdy2,blm2,ilm2);

% plot the mapping result with landmarks
figure;
patch('Faces',f1,'Vertices',map,'FaceColor','None','EdgeColor','k');
axis equal tight;
hold on;
plot(map(blm1,1),map(blm1,2),'ro','MarkerFaceColor','r');
plot(map(ilm1,1),map(ilm1,2),'ro','MarkerFaceColor','r');
title('Mapping result');
set(gca,'FontSize',15);

%% visualize the result
% rescale it for the image interpolation later
v1 = v1/18.06*1300;
v2 = v2/18.06*1300;
map = map/18.06*1300;

% flip the y-coordinates to fit the image orientation
v1(:,2) = -v1(:,2);
v2(:,2) = -v2(:,2);
map(:,2) = -map(:,2);

% get boundary information
TR1 = TriRep(f1,v1); B1 = freeBoundary(TR1); bdy_index1 = B1(:,1);
TR2 = TriRep(f2,v2); B2 = freeBoundary(TR2); bdy_index2 = B2(:,1);
TR = TriRep(f1,map); B = freeBoundary(TR); bdy_index = B(:,1);

% clear the part outside the wings
[X,Y] = meshgrid(1:1300,1:750); 
X = X(:); Y = Y(:);
newv1 = InPolygon(X,Y,v1(bdy_index1,1),v1(bdy_index1,2));
interior1 = [X(newv1),Y(newv1)];
newI1 = ones(750,1300,3);
for i = 1:length(interior1)
    newI1(interior1(i,2),interior1(i,1),:) = I1(interior1(i,2),interior1(i,1),:);
end
figure;
imshow(newI1(min(interior1(:,2)):max(interior1(:,2)),min(interior1(:,1)):max(interior1(:,1)),:));

newv2 = InPolygon(X,Y,v2(bdy_index2,1),v2(bdy_index2,2));
interior2 = [X(newv2),Y(newv2)];
newI2 = ones(750,1300,3);
for i = 1:length(interior2)
    newI2(interior2(i,2),interior2(i,1),:) = I2(interior2(i,2),interior2(i,1),:);
end
figure;
imshow(newI2(min(interior2(:,2)):max(interior2(:,2)),min(interior2(:,1)):max(interior2(:,1)),:));

% interpolation for the image
% start from target domain, find the interpolated pre-image
% otherwise easy to miss some points

% construct grid points
[X,Y] = meshgrid(floor(min(map(:,1))):ceil(max(map(:,1))),floor(min(map(:,2))):ceil(max(map(:,2)))); 
X = X(:); Y = Y(:);
newv = InPolygon(X,Y,map(bdy_index,1),map(bdy_index,2));
interior = [X(newv),Y(newv)];

% construct interpolation function
F1 = TriScatteredInterp(map(:,1:2),v1(:,1),'natural');
F2 = TriScatteredInterp(map(:,1:2),v1(:,2),'natural');

v_interp = [];
v_interp(:,1) = F1(interior(:,1),interior(:,2));
v_interp(:,2) = F2(interior(:,1),interior(:,2));
v_interp = round(v_interp);

% interpolate the image
InterpolatedImage = ones(750,1300,3);
for i = 1:length(v_interp)
    InterpolatedImage(interior(i,2),interior(i,1),:) = newI1(v_interp(i,2),v_interp(i,1),:);
end

figure;
imshow(InterpolatedImage(min(interior(:,2)):max(interior(:,2)),min(interior(:,1)):max(interior(:,1)),:));

% intensity difference
error_image = abs(InterpolatedImage-newI2);
figure;
imshow(1-error_image)
title('Teichmuller')

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comparison with other mapping methods %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Comparison: Direct overlay
error_image_direct = abs(newI1-newI2);
figure;
imshow(1-error_image_direct)
title('Direct overlay')

%% Comparison: Procrustes superimposition
[~,~,transform] = procrustes(v2([blm2;ilm2],:),v1([blm1;ilm1],:));
c = transform.c;
T = transform.T;
b = transform.b;
map = b*v1*T; map(:,1) = map(:,1)+c(1,1); map(:,2) = map(:,2)+c(1,2); 

% construct grid points
[X,Y] = meshgrid(floor(min(map(:,1))):ceil(max(map(:,1))),floor(min(map(:,2))):ceil(max(map(:,2)))); 
X = X(:); Y = Y(:);
newv = InPolygon(X,Y,map(bdy_index,1),map(bdy_index,2));
interior = [X(newv),Y(newv)];

% construct interpolation function
F1 = TriScatteredInterp(map(:,1:2),v1(:,1),'natural');
F2 = TriScatteredInterp(map(:,1:2),v1(:,2),'natural');
v_interp = [];
v_interp(:,1) = F1(interior(:,1),interior(:,2));
v_interp(:,2) = F2(interior(:,1),interior(:,2));
v_interp = round(v_interp);

% interpolate the image
InterpolatedImage = ones(750,1300,3);
for i = 1:length(v_interp)
    InterpolatedImage(interior(i,2),interior(i,1),:) = newI1(v_interp(i,2),v_interp(i,1),:);
end

% intensity difference
error_image = abs(InterpolatedImage-newI2);
figure;
imshow(1-error_image)
title('Procrustes superimposition')

%% Comparison: Conformal map
z = complex(v1([blm1;ilm1],1),v1([blm1;ilm1],2));
w = complex(v2([blm2;ilm2],1),v2([blm2;ilm2],2));
objfcn = @(v)v(1)+v(2)*z+v(3)*z.^2+v(4)*z.^3-w;
opts = optimoptions(@lsqnonlin,'Algorithm','levenberg-marquardt','Display','off');
x0 = [1;1;1;1]; % arbitrary initial guess
[vestimated,~,~,~,~] = lsqnonlin(objfcn,x0,[],[],opts);
u = complex(v1(:,1),v1(:,2));
map = vestimated(1)+vestimated(2)*u+vestimated(3)*u.^2+vestimated(4)*u.^3;
map = [real(map),imag(map)];

% construct grid points
[X,Y] = meshgrid(floor(min(map(:,1))):ceil(max(map(:,1))),floor(min(map(:,2))):ceil(max(map(:,2)))); 
X = X(:); Y = Y(:);
newv = InPolygon(X,Y,map(bdy_index,1),map(bdy_index,2));
interior = [X(newv),Y(newv)];

% construct interpolation function
F1 = TriScatteredInterp(map(:,1:2),v1(:,1),'natural');
F2 = TriScatteredInterp(map(:,1:2),v1(:,2),'natural');
v_interp = [];
v_interp(:,1) = F1(interior(:,1),interior(:,2));
v_interp(:,2) = F2(interior(:,1),interior(:,2));
v_interp = round(v_interp);

% interpolate the image
InterpolatedImage = ones(750,1300,3);
for i = 1:length(v_interp)
    InterpolatedImage(interior(i,2),interior(i,1),:) = newI1(v_interp(i,2),v_interp(i,1),:);
end

% intensity difference
error_image = abs(InterpolatedImage-newI2);
figure;
imshow(1-error_image)
title('Conformal map')

%% Comparison: Thin plate spline
tps = tpaps(v1([blm1;ilm1],:)',v2([blm2;ilm2],:)',1/2);
map = fnval(tps,v1(:,1:2)')';

% construct grid points
[X,Y] = meshgrid(floor(min(map(:,1))):ceil(max(map(:,1))),floor(min(map(:,2))):ceil(max(map(:,2)))); 
X = X(:); Y = Y(:);
newv = InPolygon(X,Y,map(bdy_index,1),map(bdy_index,2));
interior = [X(newv),Y(newv)];

% construct interpolation function
F1 = TriScatteredInterp(map(:,1:2),v1(:,1),'natural');
F2 = TriScatteredInterp(map(:,1:2),v1(:,2),'natural');
v_interp = [];
v_interp(:,1) = F1(interior(:,1),interior(:,2));
v_interp(:,2) = F2(interior(:,1),interior(:,2));
v_interp = round(v_interp);

% interpolate the image
InterpolatedImage = ones(750,1300,3);
for i = 1:length(v_interp)
    InterpolatedImage(interior(i,2),interior(i,1),:) = newI1(v_interp(i,2),v_interp(i,1),:);
end

% intensity difference
error_image = abs(InterpolatedImage-newI2);
figure;
imshow(1-error_image)
title('Thin plate spline')
