% Demo: Studying the phenotypic variation of Hawaiian Drosophila wings across species using curvature-guided, landmark-matching Teichmller mappings
%
% Copyright (c) 2018, Gary P. T. Choi
% https://scholar.harvard.edu/choi

addpath(genpath('.'));

%% Part 1: landmark-matching curvature-guided Teichmuller map
% note: running this part may take a long time
% you may skip this part and load the precomputed similarity matrix in Part 2 instead

winglist = ls('data/hawaiian_drosophila_wings/*.mat');
specimen = cell(size(winglist,1),1);
for t = 1:size(winglist,1)
    location = find(winglist(t,:) == '.');
    specimen{t} = winglist(t,1:location-1);
end
similarity = zeros(size(winglist,1),size(winglist,1));

% pairwise mapping
for id1 = 1:length(similarity)
    specimen1 = specimen{id1};
    data1 = load(['data/hawaiian_drosophila_wings/',specimen1,'.mat']);
    v1 = data1.v; f1 = data1.f; bdy1 = data1.bdy; blm1 = data1.blm; ilm1 = data1.ilm;
    parfor id2 = 1:length(similarity)
        specimen2 = specimen{id2};  
        data2 = load(['data/hawaiian_drosophila_wings/',specimen2,'.mat']);
        v2 = data2.v; f2 = data2.f; bdy2 = data2.bdy; blm2 = data2.blm; ilm2 = data2.ilm;
        
        % landmark-matching curvatured-guided Teichmuller mapping
        [map,map_mu] = curvature_guided_Teichmuller_map(v1,f1,bdy1,blm1,ilm1,v2,f2,bdy2,blm2,ilm2);
        
        % similarity score
        similarity(id1,id2) = 1-mean(abs(map_mu));
    end
end

% save('hawaiian_similarity.mat'); % save the result

%% Part 2: Statistical analysis
load('hawaiian_similarity.mat') % load the precomputed result

% plot the similarity matrix
figure;
imagesc(similarity);
colorbar
colormap bone
colormap(flipud(colormap))
caxis([min(similarity(:)) 1])
set(gca,'linewidth',2);
set(gca,'FontSize',16);
xlabel('Specimen')
ylabel('Specimen')
title('Similarity')

A = similarity;

% adaptive thresholding 
similarity_old = Inf;
threshold_parameter = 1; % 0, 0.5, 0.75, 1, 1.25, 1.5
while sum(sum(similarity-similarity_old)) ~= 0
    similarity_old = similarity;
    for i = 1:length(similarity)
        row_i = similarity(i,:);
        similarity(i,row_i>mean(row_i)+threshold_parameter*std(row_i)) = 1;
        similarity(i,row_i<=mean(row_i)+threshold_parameter*std(row_i)) = 0;
    end
    similarity = (similarity+similarity')/2;
end

% thresholded similarity
figure;
imagesc(similarity);
colorbar
colormap bone
colormap(flipud(colormap))
set(gca,'linewidth',2);
set(gca,'FontSize',16);
xlabel('Specimen')
ylabel('Specimen')
title('Thresholded similarity')

% Community detection
[community_label,~] = DetectCommunitiesBipartite(...
    [zeros(length(similarity)) similarity; similarity' zeros(length(similarity))], length(similarity), [], 0);
community_label_unique = unique(community_label);
temp = community_label;
for i = 1:length(community_label_unique)
    temp(community_label == community_label_unique(i)) = i;
end
community_label = temp;

% visualization on the MDS plane
D = ((1-A)+(1-A)')/2;
[C,~] = mdscale(D-diag(diag(D)),2); C = C';
colors = distinguishable_colors(length(community_label_unique));
colors([1,2],:) = colors([2 1],:);
figure;
hold on;
for t = 1:length(A)
    if t >= 1 && t <= 32
        s = 'o';
    elseif t >= 33 && t <= 65
        s = 's';
    elseif t >= 66 && t <= 87 
        s = '^';
    else s = 'd';
    end
    plot(C(1,t),C(2,t),['k',s],'MarkerFaceColor',colors(community_label(t),:),'LineWidth',2,'MarkerSize',13);
    text(C(1,t)+0.005,C(2,t)+0.005, num2str(t),'FontSize',20); 
end
set(gca,'linewidth',4,'FontSize',22);
xlabel('MDS Coordinate 1')
ylabel('MDS Coordinate 2')
axis([-0.18 0.23 -0.18 0.19])

community1 = find(community_label == 2); % glabriapex community
community2 = find(community_label == 1); % planitibia community
community3 = find(community_label == 3); % grimshawi community
phylogenetic1 = 1:32;   % adiasola
phylogenetic2 = 33:65;  % planitibia
phylogenetic3 = 66:87;  % glabriapex
phylogenetic4 = 88:128; % grimshawi

%% community 1
fprintf('Percentage of species from different phylogenetic groups in community 1:')
[length(intersect(phylogenetic1,community1))/length(community1);...
length(intersect(phylogenetic2,community1))/length(community1);...
length(intersect(phylogenetic3,community1))/length(community1);...
length(intersect(phylogenetic4,community1))/length(community1)]

%% community 2
fprintf('Percentage of species from different phylogenetic groups in community 2:')
[length(intersect(phylogenetic1,community2))/length(community2);...
length(intersect(phylogenetic2,community2))/length(community2);...
length(intersect(phylogenetic3,community2))/length(community2);...
length(intersect(phylogenetic4,community2))/length(community2)]

%% community 3
fprintf('Percentage of species from different phylogenetic groups in community 3:')
[length(intersect(phylogenetic1,community3))/length(community3);...
length(intersect(phylogenetic2,community3))/length(community3);...
length(intersect(phylogenetic3,community3))/length(community3);...
length(intersect(phylogenetic4,community3))/length(community3)]

%% adiastola
fprintf('Percentage of species in different communities from adiastola:')
[length(intersect(phylogenetic1,community1))/length(phylogenetic1);...
length(intersect(phylogenetic1,community2))/length(phylogenetic1);...
length(intersect(phylogenetic1,community3))/length(phylogenetic1)]

%% planitibia
fprintf('Percentage of species in different communities from planitibia:')
[length(intersect(phylogenetic2,community1))/length(phylogenetic2);...
length(intersect(phylogenetic2,community2))/length(phylogenetic2);...
length(intersect(phylogenetic2,community3))/length(phylogenetic2)]

%% glabriapex
fprintf('Percentage of species in different communities from glabriapex:')
[length(intersect(phylogenetic3,community1))/length(phylogenetic3);...
length(intersect(phylogenetic3,community2))/length(phylogenetic3);...
length(intersect(phylogenetic3,community3))/length(phylogenetic3)]

%% grimshawi
fprintf('Percentage of species in different communities from grimshawi:')
[length(intersect(phylogenetic4,community1))/length(phylogenetic4);...
length(intersect(phylogenetic4,community2))/length(phylogenetic4);...
length(intersect(phylogenetic4,community3))/length(phylogenetic4)]

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comparison with other clustering methods %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1: hierarchical clustering
% 2: Density-based clustering
% 3: Spectral clustering (n=3)
% 4: Spectral clustering (n=4)

method = 1; % 1,2,3,4
switch method
    case 1
        %%  hierarchical clustering
        Z = linkage(1-similarity);
        IDX = cluster(Z,'maxclust',4);
        IDX_temp = IDX;
        IDX_temp(IDX == 2) = 1;
        IDX_temp(IDX == 1) = 2;
        IDX = IDX_temp;
    case 2
        %% Density-based clustering
        [IDX,~] = DBSCAN(1-similarity,0.1,21);
        IDX = IDX + 1;
        % for consistent coloring
        IDX_temp = IDX;
        IDX_temp(IDX == 4) = 3;
        IDX_temp(IDX == 3) = 2;
        IDX_temp(IDX == 2) = 1;
        IDX_temp(IDX == 1) = 4;
        IDX = IDX_temp;
    case 3
        %% Spectral clustering (n=3)
        VV = GCSpectralClust2(1-similarity, 3, 5);
        IDX = VV(:,end);
        % note: the order of the colors is random
    case 4
        %% Spectral clustering (n=4)
        VV = GCSpectralClust2(1-similarity, 4, 5);
        IDX = VV(:,end);
        % note: the order of the colors is random
end

% visualization on the MDS plane 
colors = distinguishable_colors(20);
figure;
hold on;
for t = 1:length(A)
    if t >= 1 && t <= 32
        s = 'o';
    elseif t >= 33 && t <= 65
        s = 's';
    elseif t >= 66 && t <= 87 
        s = '^';
    else s = 'd';
    end
    plot(C(1,t),C(2,t),['k',s],'MarkerFaceColor',colors(IDX(t),:),'LineWidth',2,'MarkerSize',13);
    text(C(1,t)+0.005,C(2,t)+0.005, num2str(t),'FontSize',20); 
end
set(gca,'linewidth',4,'FontSize',22);
xlabel('MDS Coordinate 1')
ylabel('MDS Coordinate 2')
axis([-0.18 0.23 -0.18 0.19])